@@ -63,9 +63,6 @@
 #define MENU_IS_UP    0x04
 #define MENU_IS_DOWN  0x08
 
-#define MAX_ITEM_LABELS  3
-#define MAX_MINI_ICONS   2
-
 /*************************
  * MENU STYLE STRUCTURES *
  *************************/
@@ -87,6 +84,7 @@ typedef struct MenuFeel
   struct
   {
     unsigned is_animated : 1;
+    unsigned do_popdown_immediately : 1;
     unsigned do_popup_immediately : 1;
     unsigned do_warp_to_title : 1;
     unsigned do_popup_as_root_menu : 1;
@@ -94,8 +92,11 @@ typedef struct MenuFeel
     unsigned use_left_submenus : 1;
     unsigned use_automatic_hotkeys : 1;
   } flags;
+  int PopdownDelay10ms;
   int PopupOffsetPercent;
   int PopupOffsetAdd;
+  int PopupDelay10ms;
+  int DoubleClickTime;
   char *item_format;
   KeyCode select_on_release_key;
 } MenuFeel;
@@ -280,6 +281,9 @@ typedef struct MenuStyle
 #define ST_DO_POPUP_IMMEDIATELY(s)    ((s)->feel.flags.do_popup_immediately)
 #define MST_DO_POPUP_IMMEDIATELY(m) \
         ((m)->s->ms->feel.flags.do_popup_immediately)
+#define ST_DO_POPDOWN_IMMEDIATELY(s)    ((s)->feel.flags.do_popdown_immediately)
+#define MST_DO_POPDOWN_IMMEDIATELY(m) \
+        ((m)->s->ms->feel.flags.do_popdown_immediately)
 #define ST_DO_WARP_TO_TITLE(s)        ((s)->feel.flags.do_warp_to_title)
 #define MST_DO_WARP_TO_TITLE(m)       ((m)->s->ms->feel.flags.do_warp_to_title)
 #define ST_DO_POPUP_AS_ROOT_MENU(s)   ((s)->feel.flags.do_popup_as_root_menu)
@@ -301,6 +305,12 @@ typedef struct MenuStyle
 #define MST_POPUP_OFFSET_PERCENT(m)   ((m)->s->ms->feel.PopupOffsetPercent)
 #define ST_POPUP_OFFSET_ADD(s)        ((s)->feel.PopupOffsetAdd)
 #define MST_POPUP_OFFSET_ADD(m)       ((m)->s->ms->feel.PopupOffsetAdd)
+#define ST_POPDOWN_DELAY(s)           ((s)->feel.PopdownDelay10ms)
+#define MST_POPDOWN_DELAY(m)          ((m)->s->ms->feel.PopdownDelay10ms)
+#define ST_POPUP_DELAY(s)             ((s)->feel.PopupDelay10ms)
+#define MST_POPUP_DELAY(m)            ((m)->s->ms->feel.PopupDelay10ms)
+#define ST_DOUBLE_CLICK_TIME(s)       ((s)->feel.DoubleClickTime)
+#define MST_DOUBLE_CLICK_TIME(m)      ((m)->s->ms->feel.DoubleClickTime)
 #define ST_ITEM_FORMAT(s)             ((s)->feel.item_format)
 #define MST_ITEM_FORMAT(m)            ((m)->s->ms->feel.item_format)
 #define ST_SELECT_ON_RELEASE_KEY(s)   ((s)->feel.select_on_release_key)
@@ -320,38 +330,39 @@ struct MenuRoot; /* forward declaration */
  * get implemented. */
 typedef struct MenuItem
 {
-    struct MenuItem *next;	/* next menu item */
-    struct MenuItem *prev;	/* prev menu item */
+  struct MenuItem *next;	/* next menu item */
+  struct MenuItem *prev;	/* prev menu item */
 
-    char *label[MAX_ITEM_LABELS]; /* the strings displayed in the item */
-    unsigned short label_offset[MAX_ITEM_LABELS]; /* witdh of label[i] */
-    unsigned short label_strlen[MAX_ITEM_LABELS]; /* strlen(label[i]) */
+  char *label[MAX_MENU_ITEM_LABELS]; /* the strings displayed in the item */
+  unsigned short label_offset[MAX_MENU_ITEM_LABELS]; /* witdh of label[i] */
+  unsigned short label_strlen[MAX_MENU_ITEM_LABELS]; /* strlen(label[i]) */
 
-    Picture *picture;           /* Pixmap to show above label*/
-    Picture *lpicture[MAX_MINI_ICONS]; /* Pics to show left/right of label */
+  Picture *picture;           /* Pixmap to show above label*/
+  /* Pics to show left/right of label */
+  Picture *lpicture[MAX_MENU_ITEM_MINI_ICONS];
 
-    short y_offset;		/* y offset for item */
-    short height;		/* y height for item */
+  short y_offset;		/* y offset for item */
+  short height;		/* y height for item */
 
-    char *action;		/* action to be performed */
-    short func_type;		/* type of built in function */
-    short hotkey_coffset;	/* Hot key offset (pete@tecc.co.uk). */
-    char hotkey_column;         /* The column number the hotkey is defined in*/
-    struct
-    {
-      unsigned is_separator : 1;
-      unsigned is_title : 1;
-      unsigned is_title_centered : 1;
-      unsigned is_popup : 1;
-      unsigned is_menu : 1;
-      unsigned has_text : 1;
-      unsigned has_picture : 1;
-      unsigned has_hotkey : 1;
-      unsigned is_hotkey_automatic : 1;
-      unsigned is_selectable : 1;
-      /* temporary flags */
-      unsigned was_deselected : 1;
-    } flags;
+  char *action;		/* action to be performed */
+  short func_type;		/* type of built in function */
+  short hotkey_coffset;	/* Hot key offset (pete@tecc.co.uk). */
+  char hotkey_column;         /* The column number the hotkey is defined in*/
+  struct
+  {
+    unsigned is_separator : 1;
+    unsigned is_title : 1;
+    unsigned is_title_centered : 1;
+    unsigned is_popup : 1;
+    unsigned is_menu : 1;
+    unsigned has_text : 1;
+    unsigned has_picture : 1;
+    unsigned has_hotkey : 1;
+    unsigned is_hotkey_automatic : 1;
+    unsigned is_selectable : 1;
+    /* temporary flags */
+    unsigned was_deselected : 1;
+  } flags;
 } MenuItem;
 
 #define MI_NEXT_ITEM(i)         ((i)->next)
@@ -401,7 +412,8 @@ typedef struct MenuRootStatic
   unsigned short height;      /* height of the menu */
   unsigned short item_width;          /* width of the actual menu item */
   unsigned short sidepic_x_offset;    /* offset of the sidepic */
-  unsigned short icon_x_offset[MAX_MINI_ICONS]; /* offsets of the mini icons */
+  unsigned short icon_x_offset[MAX_MENU_ITEM_MINI_ICONS];
+                              /* offsets of the mini icons */
   unsigned short triangle_x_offset;   /* offset of the submenu triangle col */
   unsigned short item_text_x_offset;  /* offset of the actual menu item */
   unsigned short item_text_y_offset;  /* y offset for item text. */
@@ -472,6 +484,7 @@ typedef struct MenuRootDynamic
   struct MenuItem *parent_item; /* the menu item that popped this up, if any */
   Window window;                /* the window of the menu */
   MenuItem *selected_item;      /* the selected item in menu */
+  MenuItem *submenu_item;       /* item that has it's submenu mapped */
   int xanimation;               /* x distance window was moved by animation */
   /* dynamic temp flags */
   struct
@@ -502,6 +515,7 @@ typedef struct MenuRootDynamic
 #define MR_PARENT_ITEM(m)           ((m)->d->parent_item)
 #define MR_WINDOW(m)                ((m)->d->window)
 #define MR_SELECTED_ITEM(m)         ((m)->d->selected_item)
+#define MR_SUBMENU_ITEM(m)          ((m)->d->submenu_item)
 #define MR_XANIMATION(m)            ((m)->d->xanimation)
 #define MR_STORED_ITEM(m)           ((m)->d->stored_item)
 /* flags */
@@ -575,6 +589,7 @@ typedef struct
   int *pcontext;
   XEvent *eventp;
   char **ret_paction;
+  XEvent *event_propagate_to_submenu;
   MenuOptions *pops;
   struct
   {
@@ -588,6 +603,28 @@ typedef struct
   } flags;
 } MenuParameters;
 
+typedef struct
+{
+  MenuRoot *menu;
+  /* number of item labels present in the item format */
+  unsigned int used_item_labels;
+  /* same for mini icons */
+  unsigned int used_mini_icons;
+  struct
+  {
+    unsigned short label_width[MAX_MENU_ITEM_LABELS];
+    unsigned short sidepic_width;
+    unsigned short icon_width[MAX_MENU_ITEM_MINI_ICONS];
+    unsigned short picture_width;
+    unsigned short triangle_width;
+    unsigned short title_width;
+  } max;
+  struct
+  {
+    unsigned is_popup_indicator_used : 1;
+  } flags;
+} MenuSizingParameters;
+
 /* Return values for UpdateMenu, do_menu, menuShortcuts.  This is a lame
  * hack, in that "_BUTTON" is added to mean a button-release caused the
  * return-- the macros below help deal with the ugliness. */
@@ -604,6 +641,7 @@ typedef enum
   MENU_SELECTED,
   MENU_NEWITEM,
   MENU_POST,
+  MENU_UNPOST,
   MENU_TEAR_OFF,
   /* propagate the event to a different menu */
   MENU_PROPAGATE_EVENT
@@ -616,9 +654,9 @@ typedef struct
   MenuRoot *parent_menu;
   struct
   {
+    unsigned do_unpost_submenu : 1;
     unsigned is_first_item_selected : 1;
     unsigned is_key_press : 1;
-    unsigned is_menu_pinned : 1;
     unsigned is_menu_posted : 1;
   } flags;
 } MenuReturn;
@@ -629,12 +667,6 @@ typedef struct MenuInfo
   MenuRoot *all;
   struct MenuStyle *DefaultStyle;
   struct MenuStyle *LastStyle;
-  int PopupDelay10ms;
-  int DoubleClickTime;
-  struct
-  {
-    unsigned use_animated_menus : 1;
-  } flags;
 } MenuInfo;
 
 extern MenuInfo Menus;
@@ -655,22 +687,13 @@ void AnimatedMoveOfWindow(Window w,int startX,int startY,int endX, int endY,
 MenuRoot *NewMenuRoot(char *name);
 void AddToMenu(MenuRoot *, char *, char *, Bool, Bool);
 void do_menu(MenuParameters *pmp, MenuReturn *pret);
-MenuRoot *FindPopup(char *popup_name);
 char *GetMenuOptions(char *action, Window w, FvwmWindow *tmp_win,
 		     MenuRoot *mr, MenuItem *mi, MenuOptions *pops);
-void DestroyMenu(MenuRoot *mr, Bool recreate);
-void add_item_to_menu(F_CMD_ARGS);
+Bool DestroyMenu(MenuRoot *mr, Bool do_recreate, Bool is_command_request);
 void add_another_menu_item(char *action);
-void destroy_menu(F_CMD_ARGS);
-void popup_func(F_CMD_ARGS);
-void staysup_func(F_CMD_ARGS);
 void change_mr_menu_style(MenuRoot *mr, char *stylename);
-void ChangeMenuStyle(F_CMD_ARGS);
-void DestroyMenuStyle(F_CMD_ARGS);
-void SetMenuStyle(F_CMD_ARGS);
 void UpdateAllMenuStyles(void);
 void UpdateMenuColorset(int cset);
 void SetMenuCursor(Cursor cursor);
 
-
 #endif /* _MENUS_ */
