@@ -280,7 +280,7 @@ static void merge_styles(
     {
       SAFEFREE(SGET_ICON_NAME(*merged_style));
       SSET_ICON_NAME(*merged_style, (SGET_ICON_NAME(*add_style)) ?
-		     strdup(SGET_ICON_NAME(*add_style)) : NULL);
+		     safestrdup(SGET_ICON_NAME(*add_style)) : NULL);
     }
     else
     {
@@ -294,7 +294,7 @@ static void merge_styles(
     {
       SAFEFREE(SGET_MINI_ICON_NAME(*merged_style));
       SSET_MINI_ICON_NAME(*merged_style, (SGET_MINI_ICON_NAME(*add_style)) ?
-			  strdup(SGET_MINI_ICON_NAME(*add_style)) : NULL);
+			  safestrdup(SGET_MINI_ICON_NAME(*add_style)) : NULL);
     }
     else
     {
@@ -309,7 +309,7 @@ static void merge_styles(
     {
       SAFEFREE(SGET_DECOR_NAME(*merged_style));
       SSET_DECOR_NAME(*merged_style, (SGET_DECOR_NAME(*add_style)) ?
-		      strdup(SGET_DECOR_NAME(*add_style)) : NULL);
+		      safestrdup(SGET_DECOR_NAME(*add_style)) : NULL);
     }
     else
     {
@@ -323,7 +323,7 @@ static void merge_styles(
     {
       SAFEFREE(SGET_ICON_FONT(*merged_style));
       SSET_ICON_FONT(*merged_style, (SGET_ICON_FONT(*add_style)) ?
-		     strdup(SGET_ICON_FONT(*add_style)) : NULL);
+		     safestrdup(SGET_ICON_FONT(*add_style)) : NULL);
     }
     else
     {
@@ -336,7 +336,7 @@ static void merge_styles(
     {
       SAFEFREE(SGET_WINDOW_FONT(*merged_style));
       SSET_WINDOW_FONT(*merged_style, (SGET_WINDOW_FONT(*add_style)) ?
-		       strdup(SGET_WINDOW_FONT(*add_style)) : NULL);
+		       safestrdup(SGET_WINDOW_FONT(*add_style)) : NULL);
     }
     else
     {
@@ -356,7 +356,7 @@ static void merge_styles(
     {
       SAFEFREE(SGET_FORE_COLOR_NAME(*merged_style));
       SSET_FORE_COLOR_NAME(*merged_style, (SGET_FORE_COLOR_NAME(*add_style)) ?
-			   strdup(SGET_FORE_COLOR_NAME(*add_style)) : NULL);
+			   safestrdup(SGET_FORE_COLOR_NAME(*add_style)) : NULL);
     }
     else
     {
@@ -369,7 +369,7 @@ static void merge_styles(
     {
       SAFEFREE(SGET_BACK_COLOR_NAME(*merged_style));
       SSET_BACK_COLOR_NAME(*merged_style, (SGET_BACK_COLOR_NAME(*add_style)) ?
-			   strdup(SGET_BACK_COLOR_NAME(*add_style)) : NULL);
+			   safestrdup(SGET_BACK_COLOR_NAME(*add_style)) : NULL);
     }
     else
     {
@@ -424,7 +424,7 @@ static void merge_styles(
 
   /* Note, only one style cmd can define a windows iconboxes,
    * the last one encountered. */
-  if (SHAS_ICON_BOXES(&add_style->flags))
+  if (SHAS_ICON_BOXES(&add_style->flag_mask))
   {
     /* If style has iconboxes */
     /* copy it */
@@ -721,7 +721,7 @@ static Bool remove_all_of_style_from_list(char *style_ref)
 
 
 /* Remove a style. */
-void ProcessDestroyStyle(F_CMD_ARGS)
+void CMD_DestroyStyle(F_CMD_ARGS)
 {
   char *name;
 
@@ -814,7 +814,7 @@ void lookup_style(FvwmWindow *tmp_win, window_style *styles)
  * must be freed in ProcessDestroyStyle().
  *
  */
-void ProcessNewStyle(F_CMD_ARGS)
+void CMD_Style(F_CMD_ARGS)
 {
   char *line;
   char *option;
@@ -834,7 +834,7 @@ void ProcessNewStyle(F_CMD_ARGS)
   /* temp area to build name list */
   window_style *ptmpstyle;
   /* which current boxes to chain to */
-  icon_boxes *which = 0;
+  icon_boxes *which = NULL;
 
   ptmpstyle = (window_style *)safemalloc(sizeof(window_style));
   /* init temp window_style area */
@@ -894,16 +894,16 @@ void ProcessNewStyle(F_CMD_ARGS)
 	else if (StrEquals(token, "ACTIVEPLACEMENTHONORSSTARTSONPAGE"))
 	{
 	  found = True;
-	  ptmpstyle->flags.active_placement_honors_starts_on_page = 1;
-	  ptmpstyle->flag_mask.active_placement_honors_starts_on_page = 1;
-	  ptmpstyle->change_mask.active_placement_honors_starts_on_page = 1;
+	  ptmpstyle->flags.manual_placement_honors_starts_on_page = 1;
+	  ptmpstyle->flag_mask.manual_placement_honors_starts_on_page = 1;
+	  ptmpstyle->change_mask.manual_placement_honors_starts_on_page = 1;
 	}
 	else if (StrEquals(token, "ACTIVEPLACEMENTIGNORESSTARTSONPAGE"))
 	{
 	  found = True;
-	  ptmpstyle->flags.active_placement_honors_starts_on_page = 0;
-	  ptmpstyle->flag_mask.active_placement_honors_starts_on_page = 1;
-	  ptmpstyle->change_mask.active_placement_honors_starts_on_page = 1;
+	  ptmpstyle->flags.manual_placement_honors_starts_on_page = 0;
+	  ptmpstyle->flag_mask.manual_placement_honors_starts_on_page = 1;
+	  ptmpstyle->change_mask.manual_placement_honors_starts_on_page = 1;
 	}
         else if (StrEquals(token, "AllowRestack"))
         {
@@ -930,6 +930,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 	    ptmpstyle->flag_mask.use_colorset = 1;
 	    ptmpstyle->change_mask.use_colorset = 1;
           }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "Style BackColor requires color argument");
+	  }
         }
         else if (StrEquals(token, "BUTTON"))
         {
@@ -943,6 +948,12 @@ void ProcessNewStyle(F_CMD_ARGS)
             ptmpstyle->flag_mask.is_button_disabled |= (1 << butt);
             ptmpstyle->change_mask.is_button_disabled |= (1 << butt);
 	  }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "Invalid Button Style Index given");
+	  }
+
         }
         else if (StrEquals(token, "BorderWidth"))
         {
@@ -954,6 +965,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 	    ptmpstyle->flag_mask.has_border_width = 1;
 	    ptmpstyle->change_mask.has_border_width = 1;
 	  }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "Style BorderWidth requires width argument");
+	  }
         }
         else if (StrEquals(token, "BackingStore"))
         {
@@ -983,7 +999,14 @@ void ProcessNewStyle(F_CMD_ARGS)
         break;
 
       case 'c':
-        if (StrEquals(token, "CLEVERPLACEMENT"))
+	if (StrEquals(token, "CascadePlacement"))
+        {
+	  found = True;
+          ptmpstyle->flags.placement_mode = PLACE_CASCADE;
+          ptmpstyle->flag_mask.placement_mode = PLACE_MASK;
+          ptmpstyle->change_mask.placement_mode = PLACE_MASK;
+        }
+        else if (StrEquals(token, "CLEVERPLACEMENT"))
         {
 	  found = True;
           ptmpstyle->flags.placement_mode |= PLACE_CLEVER;
@@ -1011,11 +1034,13 @@ void ProcessNewStyle(F_CMD_ARGS)
 	  ptmpstyle->flag_mask.capture_honors_starts_on_page = 1;
 	  ptmpstyle->change_mask.capture_honors_starts_on_page = 1;
 	}
-        else if (StrEquals(token, "COLORSET"))
+        else if (StrEquals(token, "ColorSet"))
 	{
 	  found = True;
           *val = -1;
 	  GetIntegerArguments(rest, NULL, val, 1);
+	  if (*val < 0)
+	    *val = -1;
 	  SSET_COLORSET(*ptmpstyle, *val);
 	  AllocColorset(*val);
 	  ptmpstyle->flags.use_colorset = (*val >= 0);
@@ -1030,7 +1055,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 	  found = True;
 	  next = GetNextToken(rest, &token);
 	  if (token == NULL)
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "Color Style requires a color argument");
 	    break;
+	  }
 	  if (strncasecmp(token, "rgb:", 4) == 0)
 	  {
 	    char *s;
@@ -1082,7 +1111,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 
 	  GetNextToken(rest, &token);
 	  if (!token)
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "Color Style called with incomplete color argument.");
 	    break;
+	  }
           SAFEFREE(SGET_BACK_COLOR_NAME(*ptmpstyle));
 	  SSET_BACK_COLOR_NAME(*ptmpstyle, token);
 	  ptmpstyle->flags.has_color_back = 1;
@@ -1118,7 +1151,7 @@ void ProcessNewStyle(F_CMD_ARGS)
 	  SMSET_DO_CIRCULATE_SKIP_ICON(*ptmpstyle, 1);
 	  SCSET_DO_CIRCULATE_SKIP_ICON(*ptmpstyle, 1);
         }
-        else if (StrEquals(token, "CLICKTOFOCUS"))
+        else if (StrEquals(token, "ClickToFocus"))
         {
 	  found = True;
 	  SFSET_FOCUS_MODE(*ptmpstyle, FOCUS_CLICK);
@@ -1232,7 +1265,7 @@ void ProcessNewStyle(F_CMD_ARGS)
 	  SCSET_HAS_WINDOW_FONT(*ptmpstyle, 1);
 
 	}
-        else if (StrEquals(token, "FORECOLOR"))
+        else if (StrEquals(token, "ForeColor"))
         {
 	  found = True;
 	  GetNextToken(rest, &token);
@@ -1247,6 +1280,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 	    ptmpstyle->flag_mask.use_colorset = 1;
 	    ptmpstyle->change_mask.use_colorset = 1;
           }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "ForeColor Style needs color argument");
+	  }
         }
         else if (StrEquals(token, "FVWMBUTTONS"))
         {
@@ -1334,14 +1372,14 @@ void ProcessNewStyle(F_CMD_ARGS)
         break;
 
       case 'h':
-        if (StrEquals(token, "HINTOVERRIDE"))
+        if (StrEquals(token, "HintOverride"))
         {
 	  found = True;
 	  SFSET_HAS_MWM_OVERRIDE(*ptmpstyle, 1);
 	  SMSET_HAS_MWM_OVERRIDE(*ptmpstyle, 1);
 	  SCSET_HAS_MWM_OVERRIDE(*ptmpstyle, 1);
         }
-        else if (StrEquals(token, "HANDLES"))
+        else if (StrEquals(token, "Handles"))
         {
 	  found = True;
           ptmpstyle->flags.has_no_border = 0;
@@ -1358,6 +1396,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 	    ptmpstyle->flag_mask.has_handle_width = 1;
 	    ptmpstyle->change_mask.has_handle_width = 1;
 	  }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "HandleWidth Style needs width argument");
+ 	  }
         }
         else if (StrEquals(token, "HilightFore"))
         {
@@ -1374,6 +1417,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 	    ptmpstyle->flag_mask.use_colorset_hi = 1;
 	    ptmpstyle->change_mask.use_colorset_hi = 1;
           }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "HilightFore Style needs color argument");
+	  }
         }
         else if (StrEquals(token, "HilightBack"))
         {
@@ -1390,6 +1438,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 	    ptmpstyle->flag_mask.use_colorset_hi = 1;
 	    ptmpstyle->change_mask.use_colorset_hi = 1;
           }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "HilightBack Style needs color argument");
+	  }
         }
         else if (StrEquals(token, "HilightColorset"))
 	{
@@ -1416,7 +1469,7 @@ void ProcessNewStyle(F_CMD_ARGS)
         break;
 
       case 'i':
-        if (StrEquals(token, "ICON"))
+        if (StrEquals(token, "Icon"))
         {
 	  found = True;
 	  GetNextToken(rest, &token);
@@ -1468,6 +1521,34 @@ void ProcessNewStyle(F_CMD_ARGS)
           icon_boxes *IconBoxes = NULL;
 
 	  found = True;
+	  token = PeekToken(rest, NULL);
+	  if (!token || StrEquals(token, "none"))
+	  {
+	    /* delete icon boxes from style */
+            if (SGET_ICON_BOXES(*ptmpstyle))
+	    {
+	      remove_icon_boxes_from_style(ptmpstyle);
+	    }
+	    which = NULL;
+	    if (token)
+	    {
+	      /* disable default icon box */
+	      SFSET_DO_IGNORE_ICON_BOXES(*ptmpstyle, 1);
+	    }
+	    else
+	    {
+	      /* use default icon box */
+	      SFSET_DO_IGNORE_ICON_BOXES(*ptmpstyle, 0);
+	    }
+	    SMSET_DO_IGNORE_ICON_BOXES(*ptmpstyle, 1);
+	    SCSET_DO_IGNORE_ICON_BOXES(*ptmpstyle, 1);
+	    ptmpstyle->flags.has_icon_boxes = 0;
+	    ptmpstyle->flag_mask.has_icon_boxes = 1;
+	    ptmpstyle->change_mask.has_icon_boxes = 1;
+	    break;
+	  }
+
+	  /* otherwise try to parse the icon box */
           IconBoxes = (icon_boxes *)safemalloc(sizeof(icon_boxes));
 	  /* clear it */
           memset(IconBoxes, 0, sizeof(icon_boxes));
@@ -1523,7 +1604,7 @@ void ProcessNewStyle(F_CMD_ARGS)
                                         &width, &height);
               if (width == 0) {
 		/* zero width is invalid */
-                fvwm_msg(ERR,"ProcessNewStyle",
+                fvwm_msg(ERR,"CMD_Style",
                 "IconBox requires 4 numbers or geometry! Invalid string <%s>.",
                          token);
 		/* Drop the box */
@@ -1573,6 +1654,9 @@ void ProcessNewStyle(F_CMD_ARGS)
 	    /* new current box. save for grid */
             which = IconBoxes;
           } /* end no error */
+	  SFSET_DO_IGNORE_ICON_BOXES(*ptmpstyle, 0);
+	  SMSET_DO_IGNORE_ICON_BOXES(*ptmpstyle, 1);
+	  SCSET_DO_IGNORE_ICON_BOXES(*ptmpstyle, 1);
 	  ptmpstyle->flags.has_icon_boxes = !!(SGET_ICON_BOXES(*ptmpstyle));
 	  ptmpstyle->flag_mask.has_icon_boxes = 1;
 	  ptmpstyle->change_mask.has_icon_boxes = 1;
@@ -1582,7 +1666,7 @@ void ProcessNewStyle(F_CMD_ARGS)
           /* The grid always affects the prior iconbox */
           if (which == 0) {
 	    /* If no current box */
-            fvwm_msg(ERR,"ProcessNewStyle",
+            fvwm_msg(ERR,"CMD_Style",
                      "IconGrid must follow an IconBox in same Style command");
           } else {
 	    /* have a place to grid */
@@ -1590,7 +1674,7 @@ void ProcessNewStyle(F_CMD_ARGS)
 
 	    num = GetIntegerArguments(rest, NULL, val, 2);
             if (num != 2 || val[0] < 1 || val[1] < 1) {
-              fvwm_msg(ERR,"ProcessNewStyle",
+              fvwm_msg(ERR,"CMD_Style",
                 "IconGrid needs 2 numbers > 0. Got %d numbers. x=%d y=%d!",
                        num, val[0], val[1]);
 	      /* reset grid */
@@ -1606,13 +1690,15 @@ void ProcessNewStyle(F_CMD_ARGS)
 	      }
             } /* end bad grid */
           } /* end place to grid */
-        } else if (StrEquals(token, "ICONFILL")) {
+        }
+	else if (StrEquals(token, "ICONFILL"))
+	{
 	  found = True;
 	  /* direction to fill iconbox */
           /* The fill always affects the prior iconbox */
           if (which == 0) {
             /* If no current box */
-            fvwm_msg(ERR,"ProcessNewStyle",
+            fvwm_msg(ERR,"CMD_Style",
                      "IconFill must follow an IconBox in same Style command");
           } else {
 	    /* have a place to fill */
@@ -1626,7 +1712,7 @@ void ProcessNewStyle(F_CMD_ARGS)
 	      /* its wrong */
               if (!token)
                 token = "(none)";
-              fvwm_msg(ERR,"ProcessNewStyle",
+              fvwm_msg(ERR,"CMD_Style",
 		       "IconFill must be followed by T|B|R|L, found %s.",
                        token);
             } else {
@@ -1638,11 +1724,11 @@ void ProcessNewStyle(F_CMD_ARGS)
 		/* its wrong */
                 if (!token)
                   token = "(none)";
-                fvwm_msg(ERR,"ProcessNewStyle",
+                fvwm_msg(ERR,"CMD_Style",
                          "IconFill must be followed by T|B|R|L, found %s.",
                          token);
               } else if ((IconFill_1&ICONFILLHRZ)==(IconFill_2&ICONFILLHRZ)) {
-                fvwm_msg(ERR,"ProcessNewStyle",
+                fvwm_msg(ERR,"CMD_Style",
                  "IconFill must specify a horizontal and vertical direction.");
               } else {
 		/* Its valid! */
@@ -1656,30 +1742,54 @@ void ProcessNewStyle(F_CMD_ARGS)
             } /* end first word valid */
           } /* end have a place to fill */
         } /* end iconfill */
+        else if (StrEquals(token, "IconifyWindowGroups"))
+	{
+	  found = True;
+	  SFSET_DO_ICONIFY_WINDOW_GROUPS(*ptmpstyle, 1);
+	  SMSET_DO_ICONIFY_WINDOW_GROUPS(*ptmpstyle, 1);
+	  SCSET_DO_ICONIFY_WINDOW_GROUPS(*ptmpstyle, 1);
+	}
+        else if (StrEquals(token, "IconifyWindowGroupsOff"))
+	{
+	  found = True;
+	  SFSET_DO_ICONIFY_WINDOW_GROUPS(*ptmpstyle, 0);
+	  SMSET_DO_ICONIFY_WINDOW_GROUPS(*ptmpstyle, 1);
+	  SCSET_DO_ICONIFY_WINDOW_GROUPS(*ptmpstyle, 1);
+	}
         break;
 
       case 'j':
         break;
 
       case 'k':
+        if (StrEquals(token, "KeepWindowGroupsOnDesk"))
+	{
+	  found = True;
+	  SFSET_DO_USE_WINDOW_GROUP_HINT(*ptmpstyle, 1);
+	  SMSET_DO_USE_WINDOW_GROUP_HINT(*ptmpstyle, 1);
+	  SCSET_DO_USE_WINDOW_GROUP_HINT(*ptmpstyle, 1);
+	}
         break;
 
       case 'l':
-        if (StrEquals(token, "LENIENCE"))
+        if (StrEquals(token, "Lenience"))
         {
 	  found = True;
 	  SFSET_IS_LENIENT(*ptmpstyle, 1);
 	  SMSET_IS_LENIENT(*ptmpstyle, 1);
 	  SCSET_IS_LENIENT(*ptmpstyle, 1);
         }
-        else if (StrEquals(token, "LAYER"))
+        else if (StrEquals(token, "Layer"))
         {
 	  found = True;
-	  GetIntegerArguments(rest, NULL, val, 1);
-	  if(*val < 0)
+	  *val = -1;
+	  if (GetIntegerArguments(rest, NULL, val, 1) && *val < 0)
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+	      "Layer must be positive or zero.");
+	  }
+	  if (*val < 0)
 	  {
-	    fvwm_msg(ERR, "ProcessNewStyle",
-		     "Layer must be positive or zero." );
 	    SSET_LAYER(*ptmpstyle, -9);
 	    /* mark layer unset */
 	    ptmpstyle->flags.use_layer = 0;
@@ -1704,7 +1814,42 @@ void ProcessNewStyle(F_CMD_ARGS)
         break;
 
       case 'm':
-        if (StrEquals(token, "MWMBUTTONS"))
+	if (StrEquals(token, "ManualPlacement"))
+        {
+	  found = True;
+          ptmpstyle->flags.placement_mode = PLACE_MANUAL;
+          ptmpstyle->flag_mask.placement_mode = PLACE_MASK;
+          ptmpstyle->change_mask.placement_mode = PLACE_MASK;
+        }
+	else if (StrEquals(token, "MANUALPLACEMENTHONORSSTARTSONPAGE"))
+	{
+	  found = True;
+	  ptmpstyle->flags.manual_placement_honors_starts_on_page = 1;
+	  ptmpstyle->flag_mask.manual_placement_honors_starts_on_page = 1;
+	  ptmpstyle->change_mask.manual_placement_honors_starts_on_page = 1;
+	}
+	else if (StrEquals(token, "MANUALPLACEMENTIGNORESSTARTSONPAGE"))
+	{
+	  found = True;
+	  ptmpstyle->flags.manual_placement_honors_starts_on_page = 0;
+	  ptmpstyle->flag_mask.manual_placement_honors_starts_on_page = 1;
+	  ptmpstyle->change_mask.manual_placement_honors_starts_on_page = 1;
+	}
+	else if (StrEquals(token, "MinOverlapPlacement"))
+        {
+	  found = True;
+          ptmpstyle->flags.placement_mode = PLACE_MINOVERLAP;
+          ptmpstyle->flag_mask.placement_mode = PLACE_MASK;
+          ptmpstyle->change_mask.placement_mode = PLACE_MASK;
+        }
+	else if (StrEquals(token, "MinOverlapPercentPlacement"))
+        {
+	  found = True;
+          ptmpstyle->flags.placement_mode = PLACE_MINOVERLAPPERCENT;
+          ptmpstyle->flag_mask.placement_mode = PLACE_MASK;
+          ptmpstyle->change_mask.placement_mode = PLACE_MASK;
+        }
+        else if (StrEquals(token, "MWMBUTTONS"))
         {
 	  found = True;
 	  SFSET_HAS_MWM_BUTTONS(*ptmpstyle, 1);
@@ -1724,6 +1869,11 @@ void ProcessNewStyle(F_CMD_ARGS)
             ptmpstyle->flag_mask.has_mini_icon = 1;
             ptmpstyle->change_mask.has_mini_icon = 1;
           }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "MiniIcon Style requires an Argument");
+	  }
         }
 #endif
         else if (StrEquals(token, "MWMBORDER"))
@@ -1790,13 +1940,19 @@ void ProcessNewStyle(F_CMD_ARGS)
 	    val1 = val1 * val1_unit / 100;
 	    val2 = val2 * val2_unit / 100;
 	  }
-	  if (val1 < DEFAULT_MIN_MAX_WINDOW_WIDTH ||
-	      val1 > DEFAULT_MAX_MAX_WINDOW_WIDTH)
+	  if (val1 < DEFAULT_MIN_MAX_WINDOW_WIDTH)
+	  {
+	    val1 = DEFAULT_MIN_MAX_WINDOW_WIDTH;
+	  }
+	  if (val1 > DEFAULT_MAX_MAX_WINDOW_WIDTH || val1 <= 0)
 	  {
 	    val1 = DEFAULT_MAX_MAX_WINDOW_WIDTH;
 	  }
-	  if (val2 < DEFAULT_MIN_MAX_WINDOW_HEIGHT ||
-	      val2 > DEFAULT_MAX_MAX_WINDOW_HEIGHT)
+	  if (val2 < DEFAULT_MIN_MAX_WINDOW_HEIGHT)
+	  {
+	    val2 = DEFAULT_MIN_MAX_WINDOW_HEIGHT;
+	  }
+	  if (val2 > DEFAULT_MAX_MAX_WINDOW_HEIGHT || val2 <= 0)
 	  {
 	    val2 = DEFAULT_MAX_MAX_WINDOW_HEIGHT;
 	  }
@@ -1907,7 +2063,7 @@ void ProcessNewStyle(F_CMD_ARGS)
 	  SMSET_IS_LENIENT(*ptmpstyle, 1);
 	  SCSET_IS_LENIENT(*ptmpstyle, 1);
         }
-        else if (StrEquals(token, "NOBUTTON"))
+        else if (StrEquals(token, "NoButton"))
         {
 	  found = True;
           butt = -1;
@@ -1919,6 +2075,11 @@ void ProcessNewStyle(F_CMD_ARGS)
             ptmpstyle->flag_mask.is_button_disabled |= (1 << butt);
             ptmpstyle->change_mask.is_button_disabled |= (1 << butt);
 	  }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "NoButton Style requires an argument");
+	  }
         }
         else if (StrEquals(token, "NOOLDECOR"))
         {
@@ -2142,7 +2303,7 @@ void ProcessNewStyle(F_CMD_ARGS)
 	  }
           else
 	  {
-	    fvwm_msg(ERR,"ProcessNewStyle",
+	    fvwm_msg(ERR,"CMD_Style",
 		     "bad StartsOnDesk arg: %s", rest);
 	  }
           /**/
@@ -2182,7 +2343,7 @@ void ProcessNewStyle(F_CMD_ARGS)
 	  }
 	  if (spargs < 1 || spargs > 3)
 	  {
-	    fvwm_msg(ERR, "ProcessNewStyle",
+	    fvwm_msg(ERR, "CMD_Style",
 		     "bad StartsOnPage args: %s", rest);
 	  }
 	  else
@@ -2256,10 +2417,31 @@ void ProcessNewStyle(F_CMD_ARGS)
 	  SMSET_HAS_STIPPLED_TITLE(*ptmpstyle, 1);
 	  SCSET_HAS_STIPPLED_TITLE(*ptmpstyle, 1);
 	}
+        else if (StrEquals(token, "ScatterWindowGroups"))
+	{
+	  found = True;
+	  SFSET_DO_USE_WINDOW_GROUP_HINT(*ptmpstyle, 0);
+	  SMSET_DO_USE_WINDOW_GROUP_HINT(*ptmpstyle, 1);
+	  SCSET_DO_USE_WINDOW_GROUP_HINT(*ptmpstyle, 1);
+	}
         break;
 
       case 't':
-        if (StrEquals(token, "Title"))
+	if (StrEquals(token, "TileCascadePlacement"))
+        {
+	  found = True;
+          ptmpstyle->flags.placement_mode = PLACE_TILECASCADE;
+          ptmpstyle->flag_mask.placement_mode = PLACE_MASK;
+          ptmpstyle->change_mask.placement_mode = PLACE_MASK;
+        }
+	else if (StrEquals(token, "TileManualPlacement"))
+        {
+	  found = True;
+          ptmpstyle->flags.placement_mode = PLACE_TILEMANUAL;
+          ptmpstyle->flag_mask.placement_mode = PLACE_MASK;
+          ptmpstyle->change_mask.placement_mode = PLACE_MASK;
+        }
+        else if (StrEquals(token, "Title"))
         {
 	  found = True;
           ptmpstyle->flags.has_no_title = 0;
@@ -2330,10 +2512,15 @@ void ProcessNewStyle(F_CMD_ARGS)
 
 	    /* move forward one word */
             if (!hit) {
-              fvwm_msg(ERR,"ProcessNewStyle", "UseStyle: %s style not found",
+              fvwm_msg(ERR,"CMD_Style", "UseStyle: %s style not found",
 		       token);
             }
-          } /* if (len > 0) */
+          }
+	  else
+	  {
+	    fvwm_msg(ERR, "CMD_Style",
+		     "UseStyle needs an argument");
+   	  }
         }
         break;
 
@@ -2413,7 +2600,7 @@ void ProcessNewStyle(F_CMD_ARGS)
 
     if (found == False)
     {
-      fvwm_msg(ERR,"ProcessNewStyle", "bad style command: %s", option);
+      fvwm_msg(ERR, "CMD_Style", "Bad style option: %s", option);
       /* Can't return here since all malloced memory will be lost. Ignore rest
        * of line instead. */
 
@@ -2574,7 +2761,7 @@ void check_window_style_change(
    *   use_no_pposition
    *   use_start_on_desk
    *   use_start_on_page_for_transient
-   *   active_placement_honors_starts_on_page
+   *   manual_placement_honors_starts_on_page
    *   capture_honors_starts_on_page
    *   recapture_honors_starts_on_page
    *   use_layer
