@@ -181,18 +181,20 @@ static void unmap_window(FvwmWindow *t)
     /* suppress UnmapRequest event */
     XSelectInput(dpy, t->w, eventMask & ~StructureNotifyMask);
   }
-  if(IS_ICONIFIED(t))
+  if (IS_ICONIFIED(t))
   {
-    if(t->icon_pixmap_w != None)
+    if (t->icon_pixmap_w != None)
       XUnmapWindow(dpy,t->icon_pixmap_w);
-    if(t->icon_w != None)
+    if (t->icon_w != None)
       XUnmapWindow(dpy,t->icon_w);
   }
   else
   {
     XUnmapWindow(dpy,t->frame);
+#ifdef ICCCM2_UNMAP_WINDOW_PATCH
     /* this is required by the ICCCM2 */
     XUnmapWindow(dpy, t->w);
+#endif
     SetMapStateProp(t, IconicState);
   }
   if (ret)
@@ -214,6 +216,10 @@ static void map_window(FvwmWindow *t)
   unsigned long eventMask = 0;
   Status ret;
 
+  if (IS_SCHEDULED_FOR_DESTROY(t))
+  {
+    return;
+  }
   /*
    * Prevent the receipt of an UnmapNotify, since that would
    * cause a transition to the Withdrawn state.
@@ -237,8 +243,10 @@ static void map_window(FvwmWindow *t)
     XMapWindow(dpy, t->frame);
     XMapWindow(dpy, t->Parent);
     XMapWindow(dpy, t->decor_w);
+#ifdef ICCCM2_UNMAP_WINDOW_PATCH
     /* this is required by the ICCCM2 */
     XMapWindow(dpy, t->w);
+#endif
     SetMapStateProp(t, NormalState);
   }
   if (ret)
@@ -250,7 +258,7 @@ static void map_window(FvwmWindow *t)
 }
 
 
-void setEdgeThickness(F_CMD_ARGS)
+void CMD_EdgeThickness(F_CMD_ARGS)
 {
   int val, n;
 
@@ -269,7 +277,7 @@ void setEdgeThickness(F_CMD_ARGS)
   checkPanFrames();
 }
 
-void SetEdgeScroll(F_CMD_ARGS)
+void CMD_EdgeScroll(F_CMD_ARGS)
 {
   int val1, val2, val1_unit,val2_unit,n;
 
@@ -309,7 +317,7 @@ void SetEdgeScroll(F_CMD_ARGS)
   checkPanFrames();
 }
 
-void SetEdgeResistance(F_CMD_ARGS)
+void CMD_EdgeResistance(F_CMD_ARGS)
 {
   int val[2];
 
@@ -324,7 +332,7 @@ void SetEdgeResistance(F_CMD_ARGS)
 }
 
 
-void SetDeskSize(F_CMD_ARGS)
+void CMD_DesktopSize(F_CMD_ARGS)
 {
   int val[2];
 
@@ -376,16 +384,14 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
     return False;
   }
 
-  if((Scr.ScrollResistance >= 10000)||
-     ((HorWarpSize ==0)&&(VertWarpSize==0)))
+  if (Scr.ScrollResistance >= 10000 || (HorWarpSize == 0 && VertWarpSize==0))
   {
     is_timestamp_valid = False;
     add_time = 0;
     return False;
   }
 
-  /* need to move the viewport */
-  if(Scr.VxMax == 0 && Scr.VyMax == 0)
+  if (Scr.VxMax == 0 && Scr.VyMax == 0)
   {
     is_timestamp_valid = False;
     add_time = 0;
@@ -409,7 +415,7 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 
   do
   {
-    if(XPending(dpy) > 0 &&
+    if (XPending(dpy) > 0 &&
        (XCheckWindowEvent(
 	 dpy, Scr.PanFrameTop.win, LeaveWindowMask, &Event) ||
 	XCheckWindowEvent(
@@ -428,8 +434,8 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
     GetLocationFromEventOrQuery(dpy, Scr.Root, &Event, &x, &y);
     /* check actual pointer location since PanFrames can get buried under
        a window being moved or resized - mab */
-    if(( x >= edge_thickness )&&( x < Scr.MyDisplayWidth-edge_thickness )&&
-       ( y >= edge_thickness )&&( y < Scr.MyDisplayHeight-edge_thickness ))
+    if (x >= edge_thickness && x < Scr.MyDisplayWidth-edge_thickness &&
+        y >= edge_thickness && y < Scr.MyDisplayHeight-edge_thickness)
     {
       is_timestamp_valid = False;
       add_time = 0;
@@ -440,10 +446,10 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
       return False;
     }
     if (!fLoop && is_last_position_valid &&
-	((x - last_x) > MAX_PAGING_MOVE_DISTANCE ||
-	 (x - last_x) < -MAX_PAGING_MOVE_DISTANCE ||
-	 (y - last_y) > MAX_PAGING_MOVE_DISTANCE ||
-	 (y - last_y) < -MAX_PAGING_MOVE_DISTANCE))
+	(x - last_x > MAX_PAGING_MOVE_DISTANCE ||
+	 x - last_x < -MAX_PAGING_MOVE_DISTANCE ||
+	 y - last_y > MAX_PAGING_MOVE_DISTANCE ||
+	 y - last_y < -MAX_PAGING_MOVE_DISTANCE))
     {
       /* The pointer is moving too fast, prevent paging until it slows
        * down. Don't prevent paging when fLoop is set since we can't be
@@ -547,16 +553,18 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
     *yt = y - *delta_y;
 
   /* make sure the pointer isn't warped into the panframes */
-  if(*xl < edge_thickness) *xl = edge_thickness;
-  if(*yt < edge_thickness) *yt = edge_thickness;
-  if(*xl >= Scr.MyDisplayWidth - edge_thickness)
+  if (*xl < edge_thickness)
+    *xl = edge_thickness;
+  if (*yt < edge_thickness)
+    *yt = edge_thickness;
+  if (*xl >= Scr.MyDisplayWidth - edge_thickness)
     *xl = Scr.MyDisplayWidth - edge_thickness -1;
-  if(*yt >= Scr.MyDisplayHeight - edge_thickness)
+  if (*yt >= Scr.MyDisplayHeight - edge_thickness)
     *yt = Scr.MyDisplayHeight - edge_thickness -1;
 
   is_timestamp_valid = False;
   add_time = 0;
-  if ((*delta_x == 0)&&(*delta_y == 0))
+  if (*delta_x == 0 && *delta_y == 0)
   {
     return False;
   }
@@ -594,115 +602,148 @@ Bool HandlePaging(int HorWarpSize, int VertWarpSize, int *xl, int *yt,
  ****************************************************************************/
 void checkPanFrames(void)
 {
-  Bool wrapX = (Scr.flags.edge_wrap_x) && Scr.VxMax;
-  Bool wrapY = (Scr.flags.edge_wrap_y) && Scr.VyMax;
+  Bool do_unmap_l = False;
+  Bool do_unmap_r = False;
+  Bool do_unmap_t = False;
+  Bool do_unmap_b = False;
 
-  if(!(Scr.flags.windows_captured))
+  if (!Scr.flags.windows_captured)
     return;
 
   /* thickness of 0 means remove the pan frames */
-  if (edge_thickness == 0) {
-    if (Scr.PanFrameTop.isMapped) {
-      XUnmapWindow(dpy,Scr.PanFrameTop.win);
-      Scr.PanFrameTop.isMapped=False;
-    }
-    if (Scr.PanFrameBottom.isMapped) {
-      XUnmapWindow (dpy,Scr.PanFrameBottom.win);
-      Scr.PanFrameBottom.isMapped=False;
-    }
-    if (Scr.PanFrameLeft.isMapped) {
-      XUnmapWindow(dpy,Scr.PanFrameLeft.win);
-      Scr.PanFrameLeft.isMapped=False;
-    }
-    if (Scr.PanFrameRight.isMapped) {
-      XUnmapWindow (dpy,Scr.PanFrameRight.win);
-      Scr.PanFrameRight.isMapped=False;
-    }
-    return;
+  if (edge_thickness == 0)
+  {
+    do_unmap_l = True;
+    do_unmap_r = True;
+    do_unmap_t = True;
+    do_unmap_b = True;
   }
-
-  /* check they are the right size */
-  if (edge_thickness != last_edge_thickness) {
-    XResizeWindow (dpy, Scr.PanFrameTop.win, Scr.MyDisplayWidth,
-		   edge_thickness);
-    XResizeWindow (dpy, Scr.PanFrameLeft.win, edge_thickness,
-		   Scr.MyDisplayHeight);
-    XMoveResizeWindow (dpy, Scr.PanFrameRight.win,
-		       Scr.MyDisplayWidth - edge_thickness, 0,
-		       edge_thickness, Scr.MyDisplayHeight);
-    XMoveResizeWindow (dpy, Scr.PanFrameBottom.win,
-		       0, Scr.MyDisplayHeight - edge_thickness,
-		       Scr.MyDisplayWidth, edge_thickness);
-    last_edge_thickness = edge_thickness;
-  }
-
   /* Remove Pan frames if paging by edge-scroll is permanently or
    * temporarily disabled */
-  if(Scr.EdgeScrollY == 0)
+  if (Scr.EdgeScrollX == 0 || Scr.VxMax == 0)
   {
-    XUnmapWindow(dpy,Scr.PanFrameTop.win);
-    Scr.PanFrameTop.isMapped=False;
-    XUnmapWindow (dpy,Scr.PanFrameBottom.win);
-    Scr.PanFrameBottom.isMapped=False;
+    do_unmap_l = True;
+    do_unmap_r = True;
   }
-  if(Scr.EdgeScrollX == 0)
+  if (Scr.EdgeScrollY == 0 || Scr.VyMax == 0)
   {
-    XUnmapWindow(dpy,Scr.PanFrameLeft.win);
-    Scr.PanFrameLeft.isMapped=False;
-    XUnmapWindow (dpy,Scr.PanFrameRight.win);
-    Scr.PanFrameRight.isMapped=False;
+    do_unmap_t = True;
+    do_unmap_b = True;
+  }
+  if (Scr.Vx == 0 && !Scr.flags.edge_wrap_x)
+  {
+    do_unmap_l = True;
+  }
+  if (Scr.Vx == Scr.VxMax && !Scr.flags.edge_wrap_x)
+  {
+    do_unmap_r = True;
+  }
+  if (Scr.Vy == 0 && !Scr.flags.edge_wrap_y)
+  {
+    do_unmap_t = True;
+  }
+  if (Scr.Vy == Scr.VyMax && !Scr.flags.edge_wrap_y)
+  {
+    do_unmap_b = True;
   }
-  if((Scr.EdgeScrollX == 0)&&(Scr.EdgeScrollY == 0))
-    return;
 
-  /* LEFT, hide only if EdgeWrap is off */
-  if (Scr.Vx==0 && Scr.PanFrameLeft.isMapped && (!wrapX))
+  /*
+   * hide or show the windows
+   */
+
+  /* left */
+  if (do_unmap_l)
   {
-    XUnmapWindow(dpy,Scr.PanFrameLeft.win);
-    Scr.PanFrameLeft.isMapped=False;
+    if (Scr.PanFrameLeft.isMapped)
+    {
+      XUnmapWindow(dpy, Scr.PanFrameLeft.win);
+      Scr.PanFrameLeft.isMapped = False;
+    }
   }
-  else if ((Scr.Vx > 0 || wrapX) && Scr.PanFrameLeft.isMapped==False
-	   && Scr.EdgeScrollX)
+  else
   {
-    XMapRaised(dpy,Scr.PanFrameLeft.win);
-    Scr.PanFrameLeft.isMapped=True;
+    if (edge_thickness != last_edge_thickness)
+    {
+      XResizeWindow(
+	dpy, Scr.PanFrameLeft.win, edge_thickness, Scr.MyDisplayHeight);
+    }
+    if (!Scr.PanFrameLeft.isMapped)
+    {
+      XMapRaised(dpy, Scr.PanFrameLeft.win);
+      Scr.PanFrameLeft.isMapped = True;
+    }
   }
-  /* RIGHT, hide only if EdgeWrap is off */
-  if (Scr.Vx == Scr.VxMax && Scr.PanFrameRight.isMapped && (!wrapX))
+  /* right */
+  if (do_unmap_r)
   {
-    XUnmapWindow (dpy,Scr.PanFrameRight.win);
-    Scr.PanFrameRight.isMapped=False;
+    if (Scr.PanFrameRight.isMapped)
+    {
+      XUnmapWindow(dpy, Scr.PanFrameRight.win);
+      Scr.PanFrameRight.isMapped = False;
+    }
   }
-  else if ((Scr.Vx < Scr.VxMax || wrapX) && Scr.PanFrameRight.isMapped==False
-	   && Scr.EdgeScrollX)
+  else
   {
-    XMapRaised(dpy,Scr.PanFrameRight.win);
-    Scr.PanFrameRight.isMapped=True;
+    if (edge_thickness != last_edge_thickness)
+    {
+      XMoveResizeWindow(
+	dpy, Scr.PanFrameRight.win, Scr.MyDisplayWidth - edge_thickness, 0,
+	edge_thickness, Scr.MyDisplayHeight);
+    }
+    if (!Scr.PanFrameRight.isMapped)
+    {
+      XMapRaised(dpy, Scr.PanFrameRight.win);
+      Scr.PanFrameRight.isMapped = True;
+    }
   }
-  /* TOP, hide only if EdgeWrap is off */
-  if (Scr.Vy==0 && Scr.PanFrameTop.isMapped && (!wrapY))
+  /* top */
+  if (do_unmap_t)
   {
-    XUnmapWindow(dpy,Scr.PanFrameTop.win);
-    Scr.PanFrameTop.isMapped=False;
+    if (Scr.PanFrameTop.isMapped)
+    {
+      XUnmapWindow(dpy, Scr.PanFrameTop.win);
+      Scr.PanFrameTop.isMapped = False;
+    }
   }
-  else if ((Scr.Vy > 0 || wrapY) && Scr.PanFrameTop.isMapped==False
-	   && Scr.EdgeScrollY)
+  else
   {
-    XMapRaised(dpy,Scr.PanFrameTop.win);
-    Scr.PanFrameTop.isMapped=True;
+    if (edge_thickness != last_edge_thickness)
+    {
+      XResizeWindow(
+	dpy, Scr.PanFrameTop.win, Scr.MyDisplayWidth, edge_thickness);
+    }
+    if (!Scr.PanFrameTop.isMapped)
+    {
+      XMapRaised(dpy, Scr.PanFrameTop.win);
+      Scr.PanFrameTop.isMapped = True;
+    }
   }
-  /* BOTTOM, hide only if EdgeWrap is off */
-  if (Scr.Vy == Scr.VyMax && Scr.PanFrameBottom.isMapped && (!wrapY))
+  /* bottom */
+  if (do_unmap_b)
   {
-    XUnmapWindow (dpy,Scr.PanFrameBottom.win);
-    Scr.PanFrameBottom.isMapped=False;
+    if (Scr.PanFrameBottom.isMapped)
+    {
+      XUnmapWindow(dpy, Scr.PanFrameBottom.win);
+      Scr.PanFrameBottom.isMapped = False;
+    }
   }
-  else if ((Scr.Vy < Scr.VyMax || wrapY) && Scr.PanFrameBottom.isMapped==False
-	   && Scr.EdgeScrollY)
+  else
   {
-    XMapRaised(dpy,Scr.PanFrameBottom.win);
-    Scr.PanFrameBottom.isMapped=True;
+    if (edge_thickness != last_edge_thickness)
+    {
+      XMoveResizeWindow(
+	dpy, Scr.PanFrameBottom.win, 0, Scr.MyDisplayHeight - edge_thickness,
+	Scr.MyDisplayWidth, edge_thickness);
+    }
+    if (!Scr.PanFrameBottom.isMapped)
+    {
+      XMapRaised(dpy, Scr.PanFrameBottom.win);
+      Scr.PanFrameBottom.isMapped = True;
+    }
   }
+  last_edge_thickness = edge_thickness;
+
+  return;
 }
 
 /****************************************************************************
@@ -715,10 +756,14 @@ void checkPanFrames(void)
  ***************************************************************************/
 void raisePanFrames(void)
 {
-  if (Scr.PanFrameTop.isMapped) XRaiseWindow(dpy,Scr.PanFrameTop.win);
-  if (Scr.PanFrameLeft.isMapped) XRaiseWindow(dpy,Scr.PanFrameLeft.win);
-  if (Scr.PanFrameRight.isMapped) XRaiseWindow(dpy,Scr.PanFrameRight.win);
-  if (Scr.PanFrameBottom.isMapped) XRaiseWindow(dpy,Scr.PanFrameBottom.win);
+  if (Scr.PanFrameTop.isMapped)
+    XRaiseWindow(dpy,Scr.PanFrameTop.win);
+  if (Scr.PanFrameLeft.isMapped)
+    XRaiseWindow(dpy,Scr.PanFrameLeft.win);
+  if (Scr.PanFrameRight.isMapped)
+    XRaiseWindow(dpy,Scr.PanFrameRight.win);
+  if (Scr.PanFrameBottom.isMapped)
+    XRaiseWindow(dpy,Scr.PanFrameBottom.win);
 }
 
 /****************************************************************************
@@ -738,9 +783,8 @@ void initPanFrames(void)
   saved_thickness = edge_thickness;
   if (edge_thickness == 0) edge_thickness = 2;
 
-  attributes.event_mask =  (EnterWindowMask | LeaveWindowMask |
-			    VisibilityChangeMask);
-  valuemask=  (CWEventMask | CWCursor );
+  attributes.event_mask = XEVMASK_PANFW;
+  valuemask=  (CWEventMask | CWCursor);
 
   attributes.cursor = Scr.FvwmCursors[CRS_TOP_EDGE];
   /* I know these overlap, it's useful when at (0,0) and the top one is
@@ -796,22 +840,18 @@ void MoveViewport(int newx, int newy, Bool grab)
   int PageTop, PageLeft;
   int PageBottom, PageRight;
   int txl, txr, tyt, tyb;
+  FvwmWindow *sf = get_focus_window();
 
-  if(grab)
+  if (grab)
     MyXGrabServer(dpy);
-
-  prev_page_x = Scr.Vx;
-  prev_page_y = Scr.Vy;
-
-  if(newx > Scr.VxMax)
+  if (newx > Scr.VxMax)
     newx = Scr.VxMax;
-  if(newy > Scr.VyMax)
+  if (newy > Scr.VyMax)
     newy = Scr.VyMax;
-  if(newx <0)
+  if (newx <0)
     newx = 0;
-  if(newy <0)
+  if (newy <0)
     newy = 0;
-
   deltay = Scr.Vy - newy;
   deltax = Scr.Vx - newx;
   /*
@@ -822,13 +862,17 @@ void MoveViewport(int newx, int newy, Bool grab)
   PageRight     =  Scr.MyDisplayWidth  - deltax - 1;
   PageTop       =  0 - deltay;
   PageLeft      =  0 - deltax;
-
+  if (deltax || deltay)
+  {
+    prev_page_x = Scr.Vx;
+    prev_page_y = Scr.Vy;
+  }
   Scr.Vx = newx;
   Scr.Vy = newy;
-  BroadcastPacket(M_NEW_PAGE, 5,
-		  Scr.Vx, Scr.Vy, Scr.CurrentDesk, Scr.VxMax, Scr.VyMax);
+  BroadcastPacket(
+    M_NEW_PAGE, 5, Scr.Vx, Scr.Vy, Scr.CurrentDesk, Scr.VxMax, Scr.VyMax);
 
-  if((deltax!=0)||(deltay!=0))
+  if (deltax || deltay)
   {
 
     /*
@@ -968,6 +1012,11 @@ void MoveViewport(int newx, int newy, Bool grab)
     }
   }
   checkPanFrames();
+  if ((sf = get_focus_window()))
+  {
+    /* regrab buttons for focused window in case it is now obscured */
+    focus_grab_buttons(sf, True);
+  }
 
   /* do this with PanFrames too ??? HEDU */
   while(XCheckTypedEvent(dpy,MotionNotify,&Event))
@@ -978,17 +1027,18 @@ void MoveViewport(int newx, int newy, Bool grab)
   GNOME_SetCurrentArea();
 }
 
-/*=========================================================================
+/************************************************************************
  *
  * Unmap all windows on a desk -
  *   - Part 1 of a desktop switch
  *   - must eventually be followed by a call to MapDesk
  *   - unmaps from the bottom of the stack up
  *
- *=======================================================================*/
+ *************************************************************************/
 static void UnmapDesk(int desk, Bool grab)
 {
-  FvwmWindow  *t;
+  FvwmWindow *t;
+  FvwmWindow *sf = get_focus_window();
 
   if (grab)
   {
@@ -1002,7 +1052,7 @@ static void UnmapDesk(int desk, Bool grab)
     {
       if (t->Desk == desk)
       {
-	if (Scr.Focus == t)
+	if (sf == t)
 	{
 	  t->flags.is_focused_on_other_desk = 1;
 	  t->FocusDesk = desk;
@@ -1029,19 +1079,20 @@ static void UnmapDesk(int desk, Bool grab)
 }
 
 
-/*========================================================================
+/************************************************************************
  *
  * Map all windows on a desk -
  *   - Part 2 of a desktop switch
  *   - only use if UnmapDesk has previously been called
  *   - maps from the top of the stack down
  *
- *=======================================================================*/
+ ************************************************************************/
 static void MapDesk(int desk, Bool grab)
 {
   FvwmWindow *t;
   FvwmWindow *FocusWin = NULL;
   FvwmWindow *StickyWin = NULL;
+  FvwmWindow *sf = get_focus_window();
 
   Scr.flags.is_map_desk_in_progress = 1;
   if (grab)
@@ -1063,7 +1114,7 @@ static void MapDesk(int desk, Bool grab)
     {
       /*  If window is sticky, just update its desk (it's still mapped).  */
       t->Desk = desk;
-      if (Scr.Focus == t)
+      if (sf == t)
       {
 	StickyWin = t;
       }
@@ -1100,7 +1151,7 @@ static void MapDesk(int desk, Bool grab)
     /*  Otherwise, handle remembering the last-focused clicky window.  */
     if (FocusWin && (HAS_CLICK_FOCUS(FocusWin) || HAS_SLOPPY_FOCUS(FocusWin)))
     {
-      SetFocusWindow(FocusWin, 1);
+      ReturnFocusWindow(FocusWin, 1);
     }
     else if (FocusWin && !HAS_NEVER_FOCUS(FocusWin))
     {
@@ -1118,14 +1169,18 @@ static void MapDesk(int desk, Bool grab)
  * Move to a new desktop
  *
  *************************************************************************/
-void goto_desk_func(F_CMD_ARGS)
+void CMD_GotoDesk(F_CMD_ARGS)
 {
   goto_desk(GetDeskNumber(action));
 }
 
+void CMD_Desk(F_CMD_ARGS)
+{
+  CMD_GotoDesk(F_PASS_ARGS);
+}
+
 void goto_desk(int desk)
 {
-
   /*
     RBW - the unmapping operations are now removed to their own functions so
     they can also be used by the new GoToDeskAndPage command.
@@ -1155,7 +1210,7 @@ void goto_desk(int desk)
 
 
 
-/*=========================================================================
+/*************************************************************************
  *
  * Move to a new desktop and page at the same time.
  *   This function is designed for use by the Pager, and replaces the old
@@ -1163,8 +1218,8 @@ void goto_desk(int desk)
  *   - unmap all windows on the current desk so they don't flash when the
  *     viewport is moved, then switch the viewport, then the desk.
  *
- *=======================================================================*/
-void gotoDeskAndPage_func(F_CMD_ARGS)
+ *************************************************************************/
+void CMD_GotoDeskAndPage(F_CMD_ARGS)
 {
   int val[3];
   Bool is_new_desk;
@@ -1212,7 +1267,7 @@ void gotoDeskAndPage_func(F_CMD_ARGS)
 }
 
 
-void goto_page_func(F_CMD_ARGS)
+void CMD_GotoPage(F_CMD_ARGS)
 {
   int x;
   int y;
@@ -1234,31 +1289,37 @@ void goto_page_func(F_CMD_ARGS)
  *************************************************************************/
 void do_move_window_to_desk(FvwmWindow *tmp_win, int desk)
 {
-  if(tmp_win == NULL)
+  if (tmp_win == NULL)
     return;
 
   /*
    * Set the window's desktop, and map or unmap it as needed.
    */
   /* Only change mapping for non-sticky windows */
-  if(!(IS_ICONIFIED(tmp_win) && IS_ICON_STICKY(tmp_win)) &&
-     !IS_STICKY(tmp_win) /*&& !IS_ICON_UNMAPPED(tmp_win)*/)
+  if (!(IS_ICONIFIED(tmp_win) && IS_ICON_STICKY(tmp_win)) &&
+      !IS_STICKY(tmp_win) /*&& !IS_ICON_UNMAPPED(tmp_win)*/)
   {
-    if(tmp_win->Desk == Scr.CurrentDesk)
+    if (tmp_win->Desk == Scr.CurrentDesk)
     {
       tmp_win->Desk = desk;
+      if (tmp_win == get_focus_window())
+      {
+	DeleteFocus(0);
+      }
       unmap_window(tmp_win);
     }
     else if(desk == Scr.CurrentDesk)
     {
       tmp_win->Desk = desk;
       /* If its an icon, auto-place it */
-      if(IS_ICONIFIED(tmp_win))
+      if (IS_ICONIFIED(tmp_win))
 	AutoPlaceIcon(tmp_win);
       map_window(tmp_win);
     }
     else
+    {
       tmp_win->Desk = desk;
+    }
     BroadcastConfig(M_CONFIGURE_WINDOW,tmp_win);
   }
   GNOME_SetDeskCount();
@@ -1267,20 +1328,20 @@ void do_move_window_to_desk(FvwmWindow *tmp_win, int desk)
 }
 
 /* function with parsing of command line */
-void move_window_to_desk(F_CMD_ARGS)
+void CMD_MoveToDesk(F_CMD_ARGS)
 {
   int desk;
 
   if (DeferExecution(eventp,&w,&tmp_win,&context,CRS_SELECT,ButtonRelease))
     return;
   desk = GetDeskNumber(action);
-  if(desk == tmp_win->Desk)
+  if (desk == tmp_win->Desk)
     return;
   do_move_window_to_desk(tmp_win, desk);
 }
 
 
-void scroll(F_CMD_ARGS)
+void CMD_Scroll(F_CMD_ARGS)
 {
   int x,y;
   int val1, val2, val1_unit, val2_unit;
