@@ -22,7 +22,7 @@
 #include <libs/Module.h>
 
 static char const rcsid[] =
-  "$Id: readconfig.c,v 1.30 2000/11/21 14:33:20 domivogt Exp $";
+  "$Id: readconfig.c,v 1.32 2001/06/23 10:32:16 domivogt Exp $";
 
 /************************************************************************
  *
@@ -1047,11 +1047,11 @@ static NameType parse_format_dependencies (char *format)
      int id = manager;                                             \
      if (id == -1) {                                               \
        for (id = 0; id < globals.num_managers; id++) {             \
-	 globals.managers[id].##field = value;                     \
+	 globals.managers[id]. field = value;                      \
        }                                                           \
      }                                                             \
      else if (id < globals.num_managers) {                         \
-       globals.managers[id].##field = value;                       \
+       globals.managers[id]. field = value;                        \
      }                                                             \
      else {                                                        \
        ConsoleMessage ("Internal error in SET_MANAGER: %d\n", id); \
@@ -1578,7 +1578,7 @@ void read_in_resources (char *file)
 	  i = SortName;
 	}
 	else if (!strcasecmp (p, "namewithcase")) {
-	 i = SortNameCase;
+	  i = SortNameCase;
 	}
 	else if (!strcasecmp (p, "id")) {
 	  i = SortId;
