@@ -29,6 +29,10 @@ void InitList(struct XObj *xobj)
  XCharStruct struc;
  int minw,minh,resize=0;
  int NbVisCell,NbCell;
+#ifdef I18N_MB
+ char **ml;
+ XFontStruct **fs_list;
+#endif
 
  /* Enregistrement des couleurs et de la police */
  if (xobj->colorset >= 0) {
@@ -56,10 +60,21 @@ void InitList(struct XObj *xobj)
  xobj->gc=fvwmlib_XCreateGC(dpy,xobj->win,0,NULL);
  XSetForeground(dpy,xobj->gc,xobj->TabColor[fore]);
  XSetLineAttributes(dpy,xobj->gc,1,LineSolid,CapRound,JoinMiter);
- if ((xobj->xfont=XLoadQueryFont(dpy,xobj->font))==NULL)
-   fprintf(stderr,"Can't load font %s\n",xobj->font);
- else
-  XSetFont(dpy,xobj->gc,xobj->xfont->fid);
+
+#ifdef I18N_MB
+ if ((xobj->xfontset=GetFontSetOrFixed(dpy,xobj->font)) == NULL) {
+     fprintf(stderr, "FvwmScript: Couldn't load font. Exiting!\n");
+     exit(1);
+ }
+ XFontsOfFontSet(xobj->xfontset,&fs_list,&ml);
+ xobj->xfont = fs_list[0];
+#else
+ if ((xobj->xfont=GetFontOrFixed(dpy,xobj->font))==NULL) {
+   fprintf(stderr, "FvwmScript: Couldn't load font. Exiting!\n");
+   exit(1);
+ }
+#endif
+ XSetFont(dpy,xobj->gc,xobj->xfont->fid);
 
  /* Calcul de la taille du widget */
  /* Taille minimum: une ligne ou ascenseur visible */
@@ -155,13 +170,22 @@ void DrawCellule(struct XObj *xobj,int NbCell,int NbVisCell,int HeightCell,int a
 	{
 	    if (xobj->value==i)
 	    {
-		XSetForeground(dpy,xobj->gc,xobj->TabColor[shad]);
-		XFillRectangle(dpy,xobj->win,xobj->gc,r.x+2,
+		/* hili is better than shad.*/
+		XSetForeground(dpy,xobj->gc,xobj->TabColor[hili]);
+		XFillRectangle(dpy,xobj->win,xobj->gc,r.x+2, 
 			       r.y+(i-xobj->value2)*HeightCell+2,
 			       xobj->width-16-SbWidth-BdWidth,HeightCell-2);
 		DrawString(dpy,xobj,xobj->win,5+r.x,
 			   (i-xobj->value2)*HeightCell+asc+2+r.y,Title,
 			   strlen(Title),fore,back,shad,!xobj->flags[1]);
+		/* another solution is to invert bg and fg: */
+		/* XSetForeground(dpy,xobj->gc,xobj->TabColor[fore]); 
+		XFillRectangle(dpy,xobj->win,xobj->gc,r.x+2,
+			       r.y+(i-xobj->value2)*HeightCell+2,
+			       xobj->width-16-SbWidth-BdWidth,HeightCell-2);
+		DrawString(dpy,xobj,xobj->win,5+r.x,
+			   (i-xobj->value2)*HeightCell+asc+2+r.y,Title,
+			   strlen(Title),back,fore,hili,!xobj->flags[1]); */
 	    }
 	    else
 	    {
